package ibspan.tss.oms;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.rdf.model.Literal;
import com.hp.hpl.jena.rdf.model.Model;
import com.hp.hpl.jena.rdf.model.ModelFactory;
import com.hp.hpl.jena.rdf.model.Property;
import com.hp.hpl.jena.rdf.model.RDFNode;
import com.hp.hpl.jena.rdf.model.RSIterator;
import com.hp.hpl.jena.rdf.model.ResIterator;
import com.hp.hpl.jena.rdf.model.ReifiedStatement;
import com.hp.hpl.jena.rdf.model.Resource;
import com.hp.hpl.jena.rdf.model.Statement;
import com.hp.hpl.jena.rdf.model.StmtIterator;
import com.hp.hpl.jena.rdf.model.SimpleSelector;
import com.hp.hpl.jena.shared.Lock;
import com.hp.hpl.jena.vocabulary.RDF;
import com.hp.hpl.jena.query.*;
import com.ibm.adtech.jastor.JastorException;

import ibspan.tss.core.storage.ModelLoader;
import ibspan.tss.core.storage.ModelLoaderInitException;
import ibspan.tss.domain.ontology.restaurant.Restaurant;
import ibspan.tss.domain.ontology.restaurant.RestaurantFactory;
import ibspan.tss.domain.vocabulary.RestaurantVocabulary;
import ibspan.tss.domain.vocabulary.RestaurantsDBVocabulary;
import ibspan.tss.oms.vocabulary.ExtInfoDBVocabulary;
import ibspan.tss.oms.vocabulary.OntologySourceDBVocabulary;
import ibspan.tss.oms.ontology.ExtInfo;
import ibspan.tss.oms.ontology.OMSFactory;
import ibspan.tss.um.ontology.User;
import ibspan.tss.um.vocabulary.UsersDBVocabulary;

public class OMSManagement {

	private OntModel mRestaurantsDB;
	OMSCore omsCore;
	
	public OMSManagement() {
		omsCore = new OMSCore();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// TODO Auto-generated method stub
		OMSManagement omsManagement = new OMSManagement(); 
		try {
			//omsManagement.mRestaurantsDB = OMSModel.getInstance().getRestaurantsDBModel();
			//omsManagement.clearReificationTable();
			omsManagement.loadReificationTable();
			//omsManagement.incompleteData();
			//omsManagement.listReificationTable(); 
			//omsManagement.omsCore.createModelsForUpdate();
			//omsManagement.omsCore.compareRequestResponseModels("1201350089546");

		}
		catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	private void incompleteData() {
		
	}
	
	private void listReificationTable() {
		int i=0;
		
	    Model model = ModelFactory.createDefaultModel();
	    mRestaurantsDB.enterCriticalSection(Lock.READ);
	    try {
			for (RSIterator it = mRestaurantsDB.listReifiedStatements(); it.hasNext();)
			{
				//ReifiedStatement s = it.nextRS();
				Resource s = it.nextRS();
				Model m =  s.getModel();
				Restaurant r = RestaurantFactory.getRestaurant(s.getURI(), m);
				
				//System.out.println(s.getURI());
				
			    StmtIterator itSubj = s.listProperties(RDF.subject);
			    Statement stmtSubj = itSubj.nextStatement();
			    System.out.println("subject$" + stmtSubj.getObject().toString());
			    
			}
		} catch (JastorException e) {
			System.out.println("Problem with showing restaurant: " + e);
		} finally {
			mRestaurantsDB.leaveCriticalSection();
		}
		
	}
	
	private void loadReificationTable()
	{
		OntModel mRDB = OMSModel.getInstance().getRestaurantsDBModel();
		mRDB.enterCriticalSection(Lock.READ);
		int i = 0;
		try {
			for (Iterator<Restaurant> it = RestaurantFactory.getAllRestaurant(
					mRDB).iterator(); it.hasNext();) {
				Restaurant r = it.next();
			    for (StmtIterator it2 = r.resource().listProperties(); it2.hasNext();)
			    {
			    	Statement stmt = it2.nextStatement();
			    	String reifS = omsCore.createReifiedStmt(mRDB, stmt);
			    	if (reifS!=null)
			    		omsCore.addPrimaryExtInfoToReifStmt(reifS, "http://chefmoz.com/index.htm", stmt.getObject());
			    }
			    i++;
			}
		} catch (JastorException e) {
			throw new RuntimeException("Exception while retrieving restaurants from db: ", e);
		} finally {
			mRDB.leaveCriticalSection();
		}
		System.out.println("Reification table have been loaded");
	}

	private void clearReificationTable() throws ModelLoaderInitException 
	{
		mRestaurantsDB.enterCriticalSection(Lock.READ);
		try {
			for (Iterator<Restaurant> it = RestaurantFactory.getAllRestaurant(
					mRestaurantsDB).iterator(); it.hasNext();) {
				Restaurant r = it.next();		       
				for (StmtIterator it2 = r.resource().listProperties(); it2.hasNext();)
			    {
			    	Statement stmt = it2.nextStatement();
			    	omsCore.deleteReifiedStmt(mRestaurantsDB, stmt);
			    }
			}
		} catch (JastorException e) {
			throw new RuntimeException("Exception while retrieving restaurants from db: ", e);
		} finally {
			mRestaurantsDB.leaveCriticalSection();
		}
		System.out.println("Reification table have been cleard");
	}
	
}
