package ibspan.tss.oms.agents;

import ibspan.tss.core.semantic.Memory;

import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.Properties;
import java.util.Vector;

import jade.core.Agent;
import jade.core.AID;
import jade.core.behaviours.CyclicBehaviour;
import jade.core.behaviours.SequentialBehaviour;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import jade.util.Logger;

import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;
import com.hp.hpl.jena.shared.NotFoundException;

public class OMSNewDataAgent extends Agent {

	private static Logger logger = Logger.getMyLogger(OMSNewDataAgent.class.getName());
	
	private Vector<AID> dbAgents = new Vector<AID>();
	
	private Memory mem;
	
	private static final String OMS_CONF_FILE = "file:etc/oms-conf.properties";
	
	private final int FREQ_CHECK;
	
	public OMSNewDataAgent() {
		mem = new Memory();
		Properties props = loadProperties(OMS_CONF_FILE);
		FREQ_CHECK = Integer.valueOf(props.getProperty("omsNDAFrequencyChecking"));
	}
	
	public void setup() {
		
		SequentialBehaviour seqB = new SequentialBehaviour();
		
		Behaviour bCheckNewData = new CyclicBehaviour() {
			public void action() {
								
				logger.info(myAgent.getLocalName() + " check for new data");
				Map<String, String> mapModels = checkNewDataFromWrapper(myAgent.getAID());
				if (mapModels.isEmpty())
					logger.info(myAgent.getLocalName() + " no new data");
				else {
					
					DFAgentDescription dfd = new DFAgentDescription();
					ServiceDescription sd = new ServiceDescription();
					sd.setType("DB-query");
					dfd.addServices(sd);
			        try {
			        	DFAgentDescription[] result = DFService.search(myAgent, dfd);
			            dbAgents.clear();
			            for (int i = 0; i < result.length; ++i) {
			            	dbAgents.addElement(result[i].getName());
			            }
			        }
			        catch (FIPAException fe) {
			        	fe.printStackTrace();
			        }	
					
					logger.info(myAgent.getLocalName() + " yes new data");
					ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
					for (int i = 0; i < dbAgents.size(); ++i) {
						msg.addReceiver((AID)dbAgents.elementAt(i));
					}
					msg.setConversationId("OMSNewDataIndexing");
					try {
						msg.setContentObject((java.io.Serializable)mapModels);
					} catch (Exception e) { 
						e.printStackTrace();
					}
					myAgent.send(msg);
				}
				block(FREQ_CHECK);
			}
		};
		
		seqB.addSubBehaviour(bCheckNewData);
		
		addBehaviour(seqB);
		
	}
	
	private Map<String, String> checkNewDataFromWrapper(AID myAgent) {
		Map<String, String> mapModels = new HashMap<String, String>();
		boolean successDelete;
		String folderPath = "OMS/NewData/";
		
		File folder = new File(folderPath);
	    File[] listOfFiles = folder.listFiles();
	    for (int i = 0; i < listOfFiles.length; i++) {
	        if (listOfFiles[i].isFile()) {	        	
	    		try {
	    			OntModel mNewData = OntDocumentManager.getInstance()
	    			.getOntology("file:" + listOfFiles[i].getAbsolutePath(), OntModelSpec.OWL_MEM_RULE_INF);
	    			mapModels.put(listOfFiles[i].getName(), mem.model2rdf(mNewData));
	    			
			        File f = new File(listOfFiles[i].getAbsolutePath());
			        successDelete = f.delete();
	    			
	    		} catch (NotFoundException e) {
	    			logger.info(myAgent.getLocalName() + "Error: " + e.toString());
	    		}
	        }
	    }
		return mapModels;	
	}
	
	// Loads properties from the pointed URL.
	private static Properties loadProperties(String confURL) {
		Properties conf = null;
		try {
			java.net.URL url = new java.net.URL(confURL);
			conf = new Properties();
			conf.load(url.openStream());
		} catch (IOException e) {
			throw new RuntimeException("Cannot get properties from configuration file.", e);
		}
		return conf;
	}
}
