package ibspan.tss.oms.agents;

import ibspan.tss.core.semantic.Memory;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;
import java.util.Set;
import java.util.HashSet;
import java.io.FileNotFoundException;
import java.io.FileReader;

import com.hp.hpl.jena.ontology.OntDocumentManager;
import com.hp.hpl.jena.shared.NotFoundException;
import com.hp.hpl.jena.ontology.OntModel;
import com.hp.hpl.jena.ontology.OntModelSpec;

import jade.core.Agent;
import jade.core.AID;
import jade.core.behaviours.SequentialBehaviour;
import jade.core.behaviours.Behaviour;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.lang.acl.UnreadableException;
import jade.util.Logger;

public class OMSSearchAgent extends Agent {
	
	private static Logger logger = Logger.getMyLogger(OMSSearchAgent.class.getName());
	private AID initAgent; 
	private Vector dbAgents = new Vector();
	private Set<String> listURLs;
	private Memory mem;
	
	public OMSSearchAgent() {
		listURLs = new HashSet<String>();
		mem = new Memory();
	}
	
	protected void setup() {
		
		Object[] args = getArguments();
	    if (args != null && args.length > 0) {
	    	int i = 0;
	    	
	    	initAgent = new AID((String)args[0], AID.ISLOCALNAME);
	    	
	    	for (i = 1; i < args.length; ++i) {
	    		listURLs.add((String)args[i]);
	    	}
	    }
	    
	    SequentialBehaviour seqB = new SequentialBehaviour();
	    
	    seqB.addSubBehaviour(new OneShotBehaviour() {
	    	public void action() {
		        ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
		        msg.addReceiver((AID)initAgent);
		        msg.setContent(myAgent.getLocalName());
		        msg.setConversationId("SearchAgentCreated");
		        myAgent.send(msg);
	    	}
	    });
	    seqB.addSubBehaviour(new SearchData());
	    seqB.addSubBehaviour(new OneShotBehaviour() {
	    	public void action() {
	    		myAgent.doDelete();
	    	}
	    });
	    
	    addBehaviour(seqB);
	}
	
	private class SearchData extends Behaviour {
		private ACLMessage msg = null;
		private int step = 0;
		public void action() {
			switch (step) {
			case 0:
				MessageTemplate mt = MessageTemplate.and(
						MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
						MessageTemplate.MatchConversationId("OMSsearchData")
						);
				msg = myAgent.receive(mt);
				if (msg!=null) {
					try {
		            	FileWriter fstream = new FileWriter("OMS/SearchAgent/" 
		            			+ "request_" + msg.getOntology() + ".owl");
		            	BufferedWriter out = new BufferedWriter(fstream);
		            	// ontology model for update
		            	out.write(msg.getContentObject().toString());
		            	out.close();
					} catch (IOException e) {
						logger.info(myAgent.getLocalName() + "Error: " + e.toString());
					} catch (UnreadableException e) {
						logger.info(myAgent.getLocalName() + "Error: " + e.toString());
					}
					step=1;
				}
				else {
					block();
				}
				break;
			case 1:
				OntModel mResponse = checkDataFromWrapper(msg.getOntology());
				if (mResponse!=null) {
					ACLMessage reply = msg.createReply();
					reply.setPerformative(ACLMessage.INFORM);
					try {
						// URL of search page
						reply.setProtocol(msg.getProtocol());
						// unique value (long)
						reply.setOntology(msg.getOntology());
						// ontology model after update
						reply.setContentObject((java.io.Serializable)mem.model2rdf(mResponse));
					} catch (Exception e) { 
						e.printStackTrace(); 
					}
					myAgent.send(reply);
					step=2;
				} 
				else {
					block(1000);
				}
				break;
			}
		}
		public boolean done() {
			return step==2;
		}
	}
	
	private OntModel checkDataFromWrapper(String timeMillis) {
		OntModel mResponse = null;
		String filePath = "OMS/SearchAgent/response_" + timeMillis + ".owl";
		try {
			FileReader fr = new FileReader(filePath);
			fr.close();
		} catch (FileNotFoundException e) {
	    	return null;
		} catch (IOException e) {
	    	return null;
		}
		try {
			mResponse = OntDocumentManager.getInstance()
			.getOntology("file:" + filePath, OntModelSpec.OWL_MEM_RULE_INF);
		} catch (NotFoundException e) {
			return null;
		}
		return mResponse;	
	}
	
}
